package cn.com.qmth.examcloud.core.basic.api.controller;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.commons.web.support.ResponseStatus;
import cn.com.qmth.examcloud.commons.web.support.StatusResponse;
import cn.com.qmth.examcloud.core.basic.dao.StudentRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.StudentEntity;
import cn.com.qmth.examcloud.core.basic.service.AuthService;
import cn.com.qmth.examcloud.core.basic.service.SmsCodeService;
import cn.com.qmth.examcloud.core.basic.service.StudentService;
import cn.com.qmth.examcloud.core.basic.service.bean.LoginInfo;
import io.swagger.annotations.ApiOperation;

/**
 * 鉴权
 *
 * @author WANGWEI
 * @date 2018年5月25日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Transactional
@RestController
@RequestMapping("${$rmp.ctr.basic}/" + "auth")
public class AuthController extends ControllerSupport {

	@Autowired
	AuthService authService;

	@Autowired
	SmsCodeService smsCodeService;

	@Autowired
	StudentRepo studentRepo;

	@Autowired
	StudentService studentService;

	@ApiOperation(value = "登入", notes = "")
	@PostMapping("login")
	public Object login(@RequestBody LoginInfo loginInfo, HttpServletRequest request) {

		Boolean alwaysOK = loginInfo.getAlwaysOK();
		if (null != alwaysOK && alwaysOK) {
			setAlwaysOKResponse();
		}

		trim(loginInfo, false);

		String realIp = request.getHeader("x-forwarded-for");
		if (StringUtils.isBlank(realIp)) {
			realIp = request.getHeader("x-real-ip");
		}
		if (StringUtils.isNotBlank(realIp)) {
			loginInfo.setClientIp(realIp);
		} else {
			loginInfo.setClientIp(null);
		}

		User user = authService.login(loginInfo);

		if (null != alwaysOK && alwaysOK) {
			return new StatusResponse(ResponseStatus.OK, user);
		} else {
			return user;
		}
	}

	@ApiOperation(value = "登出", notes = "")
	@PostMapping("logout")
	public void logout() {
		User user = getAccessUser();
		authService.logout(user);
	}

	@ApiOperation(value = "获取登录用户", notes = "")
	@PostMapping("getLoginUser")
	public User getLoginUser(@RequestParam String key, @RequestParam String token) {

		if (StringUtils.isBlank(key)) {
			throw new StatusException("B-001009", "key is blank");
		}
		if (StringUtils.isBlank(token)) {
			throw new StatusException("B-001010", "token is blank");
		}

		return authService.getLoginUser(key, token);
	}

	@ApiOperation(value = "第三方机构接入", notes = "")
	@PostMapping("/thirdPartyAccess")
	public User thirdPartyAccess(HttpServletRequest request, @RequestParam Long orgId,
			@RequestParam String loginName, @RequestParam String appId,
			@RequestParam String timestamp, @RequestParam String token) {
		String realIp = request.getHeader("x-forwarded-for");
		if (StringUtils.isBlank(realIp)) {
			realIp = request.getHeader("x-real-ip");
		}
		return authService.thirdPartyAccess(orgId, loginName, appId, timestamp, token, realIp);
	}

	@ApiOperation(value = "发送验证码", notes = "")
	@PostMapping("sendVerificationCode4Student")
	public void sendVerificationCode4Student(@RequestParam(required = true) String phone,
			@RequestParam(required = true) Boolean bound) {

		Boolean hasBeBound = studentService.hasBeBound(phone);
		if (bound && !hasBeBound.equals(bound)) {
			throw new StatusException("B-002050", "手机号未被关联");
		} else if ((!bound) && hasBeBound.equals(bound)) {
			throw new StatusException("B-002050", "手机号已被关联");
		}

		smsCodeService.sendSmsCode(phone);
	}

	@ApiOperation(value = "发送验证码", notes = "")
	@PostMapping("/sendVerificationCode")
	public void sendVerificationCode(@RequestParam String phone) {
		smsCodeService.sendSmsCode(phone);
	}

	@ApiOperation(value = "绑定手机号", notes = "")
	@PostMapping("/bindSecurityPhone")
	public void bindSecurityPhone(@RequestParam String phone,
			@RequestParam String verificationCode) {

		smsCodeService.checkSmsCode(phone, verificationCode);

		User accessUser = getAccessUser();
		Long studentId = accessUser.getUserId();

		StudentEntity sb = studentRepo.findBySecurityPhone(phone);
		if (null != sb) {
			if (!sb.getId().equals(studentId)) {
				throw new StatusException("B-001052", "手机号已被其他用户绑定,请更换手机号");
			} else {
				throw new StatusException("B-001053", "手机号已绑定成功,无需重复绑定");
			}
		}

		StudentEntity s = studentRepo.findOne(studentId);
		s.setSecurityPhone(phone);
		studentRepo.saveAndFlush(s);
	}

}