package cn.com.qmth.examcloud.core.basic.api.controller;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.persistence.criteria.Predicate;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.helpers.poi.ExcelWriter;
import cn.com.qmth.examcloud.commons.base.util.PropertiesUtil;
import cn.com.qmth.examcloud.commons.web.config.SystemConfig;
import cn.com.qmth.examcloud.commons.web.enums.BooleanSelect;
import cn.com.qmth.examcloud.commons.web.helpers.page.PageInfo;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.controller.bean.StudentDomain;
import cn.com.qmth.examcloud.core.basic.base.constants.BasicConsts;
import cn.com.qmth.examcloud.core.basic.base.constants.PropKeys;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.StudentRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.StudentEntity;
import cn.com.qmth.examcloud.core.basic.service.StudentService;
import cn.com.qmth.examcloud.core.basic.service.bean.StudentInfo;
import cn.com.qmth.examcloud.task.api.DataSyncCloudService;
import cn.com.qmth.examcloud.task.api.request.SyncStudentReq;
import io.swagger.annotations.ApiOperation;

/**
 * 学生服务API Created by songyue on 17/1/14.
 */
@Transactional
@RestController
@RequestMapping("${$rmp.ctr.basic}/student")
public class StudentController extends ControllerSupport {

	@Autowired
	StudentRepo studentRepo;

	@Autowired
	StudentService studentService;

	@Autowired
	UserRepo userRepo;

	@Autowired
	OrgRepo orgRepo;

	@Autowired
	DataSyncCloudService dataSyncCloudService;

	private static final String[] EXCEL_HEADER = new String[]{"ID", "姓名", "学号", "身份证", "学习中心代码",
			"学习中心名称"};

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param name
	 * @param studentCode
	 * @param identityNumber
	 * @param curPage
	 * @param pageSize
	 * @return
	 */
	@ApiOperation(value = "查询学生分页数据", notes = "分页")
	@GetMapping("studentPage/{curPage}/{pageSize}")
	public PageInfo<StudentDomain> getStudentPage(@PathVariable Integer curPage,
			@PathVariable Integer pageSize, @RequestParam String name,
			@RequestParam String studentCode, @RequestParam String identityNumber,
			@RequestParam(required = false) Long rootOrgId,
			@RequestParam(required = false) BooleanSelect hasPhoto) {

		User accessUser = getAccessUser();
		if (null == rootOrgId) {
			rootOrgId = accessUser.getRootOrgId();
		}

		validateRootOrgIsolation(rootOrgId);

		final Long finalRootOrgId = rootOrgId;

		Specification<StudentEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.equal(root.get("rootOrgId"), finalRootOrgId));

			if (StringUtils.isNotEmpty(name)) {
				predicates.add(cb.like(root.get("name"), toSqlSearchPattern(name)));
			}
			if (StringUtils.isNotEmpty(studentCode)) {
				predicates.add(cb.like(root.get("studentCode"), toSqlSearchPattern(studentCode)));
			}
			if (StringUtils.isNotEmpty(identityNumber)) {
				predicates.add(
						cb.like(root.get("identityNumber"), toSqlSearchPattern(identityNumber)));
			}
			if (null != hasPhoto) {
				Boolean hasPhotoBoolean = hasPhoto.getBoolean();
				if (null != hasPhotoBoolean) {
					if (hasPhotoBoolean) {
						predicates.add(cb.isNotNull(root.get("photoPath")));
					} else {
						predicates.add(cb.isNull(root.get("photoPath")));
					}
				}
			}
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		PageRequest pageRequest = new PageRequest(curPage, pageSize,
				new Sort(Direction.DESC, "updateTime"));

		Page<StudentEntity> studentList = studentRepo.findAll(specification, pageRequest);

		Iterator<StudentEntity> iterator = studentList.iterator();

		List<StudentDomain> studentDomainList = Lists.newArrayList();

		String urlPrefix = PropertiesUtil.getString(PropKeys.STUDENT_PHOTO_URL_PREFIX);
		if (StringUtils.isBlank(urlPrefix)) {
			throw new StatusException("B-560111",
					"property[$studentPhoto.url.prefix] is not configured");
		}

		while (iterator.hasNext()) {
			StudentEntity next = iterator.next();
			StudentDomain bean = new StudentDomain();
			bean.setId(next.getId());
			bean.setName(next.getName());
			bean.setRootOrgId(next.getRootOrgId());
			bean.setUpdateTime(next.getUpdateTime());
			bean.setCreationTime(next.getCreationTime());
			bean.setEnable(next.getEnable());
			bean.setStudentCode(next.getStudentCode());
			bean.setIdentityNumber(next.getIdentityNumber());
			if (StringUtils.isNotBlank(next.getPhotoPath())) {
				bean.setPhotoPath(urlPrefix + next.getPhotoPath());
			}
			bean.setPhoneNumber(next.getPhoneNumber());
			bean.setSecurityPhone(next.getSecurityPhone());
			bean.setRemark(next.getRemark());

			bean.setOrgId(next.getOrgId());
			if (null != bean.getOrgId()) {
				OrgEntity org = orgRepo.findOne(Long.valueOf(bean.getOrgId()));
				if (null != org) {
					bean.setOrgName(org.getName());
				}
			}

			studentDomainList.add(bean);
		}

		PageInfo<StudentDomain> ret = new PageInfo<StudentDomain>();
		ret.setList(studentDomainList);
		ret.setTotal(studentList.getTotalElements());
		return ret;

	}

	@ApiOperation(value = "导出学生", notes = "")
	@GetMapping("export")
	public void exportStudents(@RequestParam String name, @RequestParam String studentCode,
			@RequestParam String identityNumber, @RequestParam(required = false) Long rootOrgId,
			@RequestParam(required = false) BooleanSelect hasPhoto) {

		User accessUser = getAccessUser();
		if (null == rootOrgId) {
			rootOrgId = accessUser.getRootOrgId();
		}

		validateRootOrgIsolation(rootOrgId);

		final Long finalRootOrgId = rootOrgId;

		Specification<StudentEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.equal(root.get("rootOrgId"), finalRootOrgId));

			if (StringUtils.isNotEmpty(name)) {
				predicates.add(cb.like(root.get("name"), toSqlSearchPattern(name)));
			}
			if (StringUtils.isNotEmpty(studentCode)) {
				predicates.add(cb.like(root.get("studentCode"), toSqlSearchPattern(studentCode)));
			}
			if (StringUtils.isNotEmpty(identityNumber)) {
				predicates.add(
						cb.like(root.get("identityNumber"), toSqlSearchPattern(identityNumber)));
			}
			if (null != hasPhoto) {
				Boolean hasPhotoBoolean = hasPhoto.getBoolean();
				if (null != hasPhotoBoolean) {
					if (hasPhotoBoolean) {
						predicates.add(cb.isNotNull(root.get("photoPath")));
					} else {
						predicates.add(cb.isNull(root.get("photoPath")));
					}
				}
			}
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		long count = studentRepo.count(specification);
		if (100000 < count) {
			throw new StatusException("B-520200", "数据量超过100000,无法导出");
		}

		List<StudentEntity> studentList = studentRepo.findAll(specification,
				new Sort(Direction.DESC, "updateTime"));

		List<Object[]> datas = Lists.newArrayList();

		for (StudentEntity cur : studentList) {
			String orgCode = null;
			String orgName = null;
			if (null != cur.getOrgId()) {
				OrgEntity org = orgRepo.findOne(Long.valueOf(cur.getOrgId()));
				if (null != org) {
					orgCode = org.getCode();
					orgName = org.getName();
				}
			}
			datas.add(new Object[]{cur.getId(), cur.getName(), cur.getStudentCode(),
					cur.getIdentityNumber(), orgCode, orgName});
		}

		String filePath = SystemConfig.getTempDataDir() + File.separator
				+ System.currentTimeMillis() + ".xlsx";
		File file = new File(filePath);

		ExcelWriter.write(EXCEL_HEADER, new Class[]{Long.class, String.class, String.class,
				String.class, String.class, String.class}, datas, new File(filePath));

		exportFile("学生-" + getRootOrgId() + ".xlsx", file);

		FileUtils.deleteQuietly(file);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param id
	 * @return
	 */
	@ApiOperation(value = "按ID查询学生", notes = "ID查询")
	@GetMapping("{id}")
	public StudentEntity getStudentById(@PathVariable Long id) {
		StudentEntity s = studentRepo.findOne(id);
		return s;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param id
	 * @return
	 */
	@ApiOperation(value = "按ID删除学生", notes = "删除")
	@DeleteMapping("{id}")
	public Long deleteStudent(@PathVariable Long id) {
		studentRepo.delete(id);
		return id;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param ids
	 * @return
	 */
	@ApiOperation(value = "启用学生")
	@PutMapping("enable/{ids}")
	public List<String> enableStudent(@PathVariable String ids) {
		List<Long> studentIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		List<String> ret = Lists.newArrayList();
		for (Long cur : studentIds) {
			StudentEntity s = studentRepo.findOne(cur);
			s.setEnable(true);
			studentRepo.save(s);
			ret.add(s.getId() + ":" + s.getName());
		}
		return ret;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param ids
	 * @return
	 */
	@ApiOperation(value = "禁用学生")
	@PutMapping("disable/{ids}")
	public List<String> disableStudent(@PathVariable String ids) {
		List<Long> studentIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		List<String> ret = Lists.newArrayList();
		for (Long cur : studentIds) {
			StudentEntity s = studentRepo.findOne(cur);
			s.setEnable(false);
			studentRepo.save(s);
			ret.add(s.getId() + ":" + s.getName());
		}
		return ret;
	}

	@ApiOperation(value = "重置用户密码", notes = "重置密码")
	@PutMapping("resetPass/{ids}")
	public void resetPassword(@PathVariable String ids) {
		List<Long> studentIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long cur : studentIds) {
			StudentEntity s = studentRepo.findOne(cur);
			if (null == s) {
				throw new StatusException("B-450110", "学生不存在");
			}
			String identityNumber = s.getIdentityNumber();
			if (StringUtils.isNotEmpty(identityNumber)
					&& identityNumber.matches("[0-9a-zA-Z]{6,}")) {
				s.setPassword(StringUtils.substring(identityNumber, -6, identityNumber.length()));
			} else {
				s.setPassword(BasicConsts.DEFAULT_PASSWORD);
			}
			studentRepo.save(s);
		}
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param newPassword
	 * @param request
	 * @return
	 */
	@ApiOperation(value = "修改学生密码", notes = "修改密码")
	@PutMapping("password/direct")
	public Long updateStudentPassword(@RequestParam("newPassword") String newPassword) {
		User accessUser = getAccessUser();
		if (StringUtils.isEmpty(newPassword)) {
			throw new StatusException("B-450201", "新密码为空");
		}

		if (!newPassword.matches("[a-zA-Z0-9]{6,18}")) {
			throw new StatusException("B-450203", "密码必须是6至18位字母或数字");
		}

		StudentEntity s = studentRepo.findOne(accessUser.getUserId());
		s.setPassword(newPassword);
		studentRepo.save(s);
		return s.getId();
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param password
	 * @param newPassword
	 * @param request
	 * @return
	 */
	@ApiOperation(value = "修改学生密码", notes = "修改密码")
	@PutMapping("password")
	public Long updateStudentPassword(@RequestParam("password") String password,
			@RequestParam("newPassword") String newPassword) {
		User accessUser = getAccessUser();
		if (StringUtils.isEmpty(password)) {
			throw new StatusException("B-450110", "旧密码为空");
		}
		if (StringUtils.isEmpty(newPassword)) {
			throw new StatusException("B-450111", "新密码为空");
		}
		if (!newPassword.matches("[a-zA-Z0-9]{6,18}")) {
			throw new StatusException("B-450111", "密码必须是6至18位字母或数字");
		}

		StudentEntity s = studentRepo.findOne(accessUser.getUserId());

		if (StringUtils.isNotBlank(s.getPassword()) && !s.getPassword().equals(password)) {
			throw new StatusException("B-450111", "当前密码错误");
		}
		s.setPassword(newPassword);
		studentRepo.save(s);
		return s.getId();
	}

	/**
	 * 查询学生
	 *
	 * @author WANGWEI
	 * @param identityNumber
	 * @param studentCode
	 * @param securityPhone
	 * @return
	 */
	@ApiOperation(value = "查询学生")
	@GetMapping("getStudentInfoBySession")
	public StudentInfo getStudentInfoBySession() {
		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();
		Long studentId = accessUser.getUserId();
		StudentInfo studentInfo = studentService.getStudentInfo(rootOrgId, studentId, null, null,
				null);
		return studentInfo;
	}

	@ApiOperation(value = "查询学生")
	@GetMapping("getStudentInfo")
	public StudentInfo getStudentInfo(@RequestParam(required = false) Long studentId,
			@RequestParam(required = false) String identityNumber,
			@RequestParam(required = false) String studentCode,
			@RequestParam(required = false) String securityPhone) {
		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();
		StudentInfo studentInfo = studentService.getStudentInfo(rootOrgId, studentId,
				identityNumber, studentCode, securityPhone);
		return studentInfo;
	}

	@ApiOperation(value = "解绑学号", notes = "")
	@PutMapping("unbindStudentCode/{ids}")
	public void unbindStudentCode(@PathVariable String ids) {
		List<Long> studentIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long cur : studentIds) {
			StudentEntity s = studentRepo.findOne(cur);
			if (null == s) {
				throw new StatusException("B-450110", "学生不存在");
			}
			validateRootOrgIsolation(s.getRootOrgId());

			s.setStudentCode(null);
			studentRepo.save(s);
		}

		for (Long cur : studentIds) {
			StudentEntity saved = studentRepo.findOne(cur);
			SyncStudentReq req = new SyncStudentReq();
			req.setEnable(saved.getEnable());
			req.setId(saved.getId());
			req.setIdentityNumber(saved.getIdentityNumber());
			req.setName(saved.getName());

			req.setOrgId(saved.getOrgId());
			OrgEntity org = orgRepo.findOne(saved.getOrgId());
			req.setOrgCode(org.getCode());
			req.setOrgName(org.getName());

			req.setPhoneNumber(saved.getPhoneNumber());
			req.setPhotoPath(saved.getPhotoPath());
			req.setRootOrgId(saved.getRootOrgId());
			req.setSecurityPhone(saved.getSecurityPhone());
			req.setStudentCode(saved.getStudentCode());

			req.setSyncType("update");

			dataSyncCloudService.syncStudent(req);
		}

	}

	@ApiOperation(value = "解绑学号", notes = "")
	@PutMapping("unbindStudentCode")
	public List<Long> unbindStudentCodeByStudentCode(
			@RequestParam(required = true) String studentCode,
			@RequestParam(required = false) String identityNumber,
			@RequestParam(required = true) Long rootOrgId) {

		if (null == rootOrgId) {
			rootOrgId = getRootOrgId();
		} else {
			OrgEntity rootOrg = orgRepo.findOne(rootOrgId);
			if (null == rootOrg) {
				throw new StatusException("B-450110", "rootOrgId is wrong");
			}
			validateRootOrgIsolation(rootOrgId);
		}

		if (StringUtils.isBlank(studentCode)) {
			throw new StatusException("B-160002", "studentCode is blank");
		}

		List<Long> studentIdList = studentService.unbindStudentCode(rootOrgId, studentCode,
				identityNumber);

		return studentIdList;
	}

	@ApiOperation(value = "解绑安全手机", notes = "")
	@PutMapping("unbindSecurityPhone/{ids}")
	public void unbindSecurityPhone(@PathVariable String ids) {
		List<Long> studentIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long cur : studentIds) {
			StudentEntity s = studentRepo.findOne(cur);
			if (null == s) {
				throw new StatusException("B-450110", "学生不存在");
			}
			validateRootOrgIsolation(s.getRootOrgId());

			s.setSecurityPhone(null);
			studentRepo.save(s);
		}

		for (Long cur : studentIds) {
			StudentEntity saved = studentRepo.findOne(cur);
			SyncStudentReq req = new SyncStudentReq();
			req.setEnable(saved.getEnable());
			req.setId(saved.getId());
			req.setIdentityNumber(saved.getIdentityNumber());
			req.setName(saved.getName());

			req.setOrgId(saved.getOrgId());
			OrgEntity org = orgRepo.findOne(saved.getOrgId());
			req.setOrgCode(org.getCode());
			req.setOrgName(org.getName());

			req.setPhoneNumber(saved.getPhoneNumber());
			req.setPhotoPath(saved.getPhotoPath());
			req.setRootOrgId(saved.getRootOrgId());
			req.setSecurityPhone(saved.getSecurityPhone());
			req.setStudentCode(saved.getStudentCode());

			req.setSyncType("update");

			dataSyncCloudService.syncStudent(req);
		}

	}

}
