package cn.com.qmth.examcloud.core.basic.service.impl;

import org.apache.commons.lang3.RandomUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.PropertiesUtil;
import cn.com.qmth.examcloud.core.basic.base.constants.PropKeys;
import cn.com.qmth.examcloud.core.basic.service.SmsCodeService;
import cn.com.qmth.examcloud.exchange.inner.api.SendSmsCloudService;
import cn.com.qmth.examcloud.exchange.inner.api.request.CheckSmsCodeReq;
import cn.com.qmth.examcloud.exchange.inner.api.request.SendSmsReq;
import cn.com.qmth.examcloud.exchange.inner.api.response.CheckIdentifyingCodeResp;

@Service
public class SmsCodeServiceImpl implements SmsCodeService {

	@Autowired
	SendSmsCloudService sendSmsCloudService;

	@Override
	public void sendSmsCode(String phone) {
		SendSmsReq req = new SendSmsReq();
		req.setPhone(phone);
		int code = 1000 + RandomUtils.nextInt(1, 9999);
		req.setCode(String.valueOf(code));

		int smsEffectivetime = PropertiesUtil.getInt(PropKeys.SEND_VERIFICATION_CODE_EFFECTIVE_TIME,
				120);
		int smsIntervalSeconds = PropertiesUtil
				.getInt(PropKeys.SEND_VERIFICATION_CODE_INTERVAL_SECONDS, 60);

		String smsSign = PropertiesUtil.getString(PropKeys.SEND_VERIFICATION_CODE_SIGN);
		String smsTemplatecode = PropertiesUtil
				.getString(PropKeys.SEND_VERIFICATION_CODE_TEMPLATE_CODE);

		if (StringUtils.isBlank(smsSign)) {
			throw new StatusException("B-001060", "签名未配置");
		}
		if (StringUtils.isBlank(smsTemplatecode)) {
			throw new StatusException("B-001061", "模板未配置");
		}

		req.setEffectiveTime(smsEffectivetime);
		req.setIntervalSeconds(smsIntervalSeconds);
		req.setSign(smsSign);
		req.setTemplateCode(smsTemplatecode);
		sendSmsCloudService.sendIdentifyingCode(req);
	}

	@Override
	public void checkSmsCode(String phone, String smsCode) {

		CheckSmsCodeReq req = new CheckSmsCodeReq();
		req.setCode(smsCode);
		req.setPhone(phone);
		CheckIdentifyingCodeResp resp = sendSmsCloudService.checkIdentifyingCode(req);
		if (!resp.getSuccess()) {
			throw new StatusException("B-001050", "验证码错误");
		}
	}

}
