package cn.com.qmth.examcloud.core.basic.service.impl;

import java.util.List;

import javax.transaction.Transactional;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.PropertiesUtil;
import cn.com.qmth.examcloud.commons.base.util.UrlUtil;
import cn.com.qmth.examcloud.core.basic.base.constants.BasicConsts;
import cn.com.qmth.examcloud.core.basic.base.constants.PropKeys;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.StudentRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.StudentEntity;
import cn.com.qmth.examcloud.core.basic.service.StudentService;
import cn.com.qmth.examcloud.core.basic.service.UserService;
import cn.com.qmth.examcloud.core.basic.service.bean.StudentInfo;
import cn.com.qmth.examcloud.task.api.DataSyncCloudService;
import cn.com.qmth.examcloud.task.api.request.SyncStudentReq;

/**
 * 学生服务类 Created by songyue on 17/1/14.
 */
@Service
public class StudentServiceImpl implements StudentService {

	@Autowired
	StudentRepo studentRepo;

	@Autowired
	UserRepo userRepo;

	@Autowired
	UserService userService;

	@Autowired
	OrgRepo orgRepo;

	@Autowired
	OrgServiceImpl orgService;

	@Autowired
	DataSyncCloudService dataSyncCloudService;

	/*
	 * 实现
	 *
	 * @author WANGWEI
	 * 
	 * @see cn.com.qmth.examcloud.core.basic.service.StudentService#
	 * insertOrUpdateStudent(cn.com.qmth.examcloud.core.basic.service.bean.
	 * StudentInfo)
	 */
	@Override
	@Transactional
	public StudentEntity saveStudent(StudentInfo studentInfo) {
		Long rootOrgId = studentInfo.getRootOrgId();
		OrgEntity rootOrg = orgRepo.findOne(rootOrgId);

		if (null == rootOrg || null != rootOrg.getParentId()) {
			throw new StatusException("B-160001", "顶级机构错误");
		}

		OrgEntity org = null;
		if (null != studentInfo.getOrgId()) {
			org = orgRepo.findOne(studentInfo.getOrgId());
			if (null == org) {
				throw new StatusException("B-160050", "orgId is wrong");
			}
			if (!org.getRootId().equals(rootOrgId)) {
				throw new StatusException("B-160051", "orgId is wrong");
			}
			if (null == org.getParentId()) {
				throw new StatusException("B-160052", "orgId is wrong");
			}
			if (!org.getParentId().equals(rootOrgId)) {
				throw new StatusException("B-160053", "orgId is wrong");
			}
		} else if (StringUtils.isNotBlank(studentInfo.getOrgCode())) {
			org = orgRepo.findByRootIdAndCode(rootOrgId, studentInfo.getOrgCode());
			if (null == org) {
				String orgName = studentInfo.getOrgName();
				if (StringUtils.isBlank(orgName)) {
					throw new StatusException("B-160003", "orgName is blank");
				}
				org = new OrgEntity();
				org.setParentId(rootOrgId);
				org.setCode(studentInfo.getOrgCode());
				org.setName(orgName);
				org.setEnable(true);
				org.setRootId(rootOrgId);
				org = orgRepo.save(org);
			}
		} else {
			throw new StatusException("B-160082", "orgId,orgCode can not be all null");
		}

		String identityNumber = studentInfo.getIdentityNumber();
		if (StringUtils.isBlank(identityNumber)) {
			throw new StatusException("B-160012", "身份证号不能为空");
		}

		StudentEntity student = studentRepo.findByIdentityNumberAndRootOrgId(identityNumber,
				rootOrgId);

		String studentCode = studentInfo.getStudentCode();
		if (StringUtils.isNotBlank(studentCode)) {
			StudentEntity studentByCode = studentRepo.findByStudentCodeAndRootOrgId(studentCode,
					rootOrgId);
			if (null != studentByCode
					&& (!studentByCode.getIdentityNumber().equalsIgnoreCase(identityNumber))) {
				throw new StatusException("B-160008", "学号被占用. 学号: " + studentCode);
			}

			if (null != student && null != student.getStudentCode()
					&& (!studentCode.equalsIgnoreCase(student.getStudentCode()))) {
				throw new StatusException("B-160005", "身份证号已关联学号");
			}
		}
		long updateTime = 0L;
		if (null != student) {
			if (null != student.getUpdateTime()) {
				updateTime = student.getUpdateTime().getTime();
			}
			if (null == student.getEnable()) {
				student.setEnable(true);
			}
		} else {
			student = new StudentEntity();
			student.setEnable(true);
			if (identityNumber.matches("[0-9a-zA-Z]{6,}")) {
				student.setPassword(
						StringUtils.substring(identityNumber, -6, identityNumber.length()));
			} else {
				student.setPassword(BasicConsts.DEFAULT_PASSWORD);
			}
		}

		student.setRootOrgId(rootOrgId);
		student.setIdentityNumber(identityNumber);
		student.setOrgId(org.getId());
		if (StringUtils.isNotBlank(studentCode)) {
			student.setStudentCode(studentCode);
		}
		if (null != studentInfo.getName()) {
			if (StringUtils.isBlank(studentInfo.getName())) {
				throw new StatusException("B-160006", "姓名不能为空");
			}
			student.setName(studentInfo.getName());
		}
		if (null != studentInfo.getPhotoPath()) {
			student.setPhotoPath(studentInfo.getPhotoPath());
		}
		if (null != studentInfo.getPhoneNumber()) {
			student.setPhoneNumber(studentInfo.getPhoneNumber());
		}
		if (null != studentInfo.getRemark()) {
			student.setRemark(studentInfo.getRemark());
		}
		if (null != studentInfo.getEnable()) {
			student.setEnable(studentInfo.getEnable());
		}
		StudentEntity saved = studentRepo.saveAndFlush(student);

		// 同步操作
		if (updateTime != saved.getUpdateTime().getTime()) {
			SyncStudentReq req = new SyncStudentReq();
			req.setEnable(saved.getEnable());
			req.setId(saved.getId());
			req.setIdentityNumber(saved.getIdentityNumber());
			req.setName(saved.getName());
			req.setOrgCode(org.getCode());
			req.setOrgId(org.getId());
			req.setOrgName(org.getName());
			req.setPhoneNumber(saved.getPhoneNumber());
			req.setPhotoPath(saved.getPhotoPath());
			req.setRootOrgId(saved.getRootOrgId());
			req.setSecurityPhone(saved.getSecurityPhone());
			req.setStudentCode(saved.getStudentCode());

			req.setSyncType("update");

			dataSyncCloudService.syncStudent(req);
		}

		return saved;
	}

	/*
	 * 实现
	 *
	 * @author WANGWEI
	 * 
	 * @see
	 * cn.com.qmth.examcloud.core.basic.service.StudentService#getStudentInfo(
	 * java.lang.Long, java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override
	public StudentInfo getStudentInfo(Long rootOrgId, Long studentId, String identityNumber,
			String studentCode, String securityPhone) {

		if (null == rootOrgId) {
			throw new StatusException("B-160250", "rootOrgId is null");
		}

		StudentEntity s = null;
		int count = 0;
		if (null != studentId) {
			count++;
			s = studentRepo.findOne(studentId);
		}
		if (StringUtils.isNotBlank(identityNumber)) {
			count++;
			s = studentRepo.findByIdentityNumberAndRootOrgId(identityNumber, rootOrgId);
		}
		if (StringUtils.isNotBlank(studentCode)) {
			count++;
			s = studentRepo.findByStudentCodeAndRootOrgId(studentCode, rootOrgId);
		}
		if (StringUtils.isNotBlank(securityPhone)) {
			count++;
			s = studentRepo.findBySecurityPhoneAndRootOrgId(securityPhone, rootOrgId);
		}

		if (count > 1) {
			throw new StatusException("B-160210",
					"参数过多,只需要[studentId,identityNumber,studentCode,securityPhone]中的一个");
		}

		if (null == s) {
			throw new StatusException("B-160211", "学生不存在");
		}

		StudentInfo info = new StudentInfo();
		info.setId(s.getId());
		info.setEnable(s.getEnable());
		info.setIdentityNumber(s.getIdentityNumber());
		info.setName(s.getName());
		info.setOrgId(s.getOrgId());
		if (null != s.getOrgId()) {
			OrgEntity org = orgRepo.findOne(s.getOrgId());
			if (null != org) {
				info.setOrgCode(org.getCode());
				info.setOrgName(org.getName());
			}
		}
		info.setPhoneNumber(s.getPhoneNumber());

		if (StringUtils.isNotBlank(s.getPhotoPath())) {
			String basePath = PropertiesUtil.getString(PropKeys.STUDENT_PHOTO_URL_PREFIX);
			if (StringUtils.isBlank(basePath)) {
				throw new StatusException("B-160212", "照片根路径未配置");
			}
			String path = UrlUtil.joinUrl(basePath, s.getPhotoPath());
			info.setPhotoPath(path);
		}

		info.setRemark(s.getRemark());
		info.setRootOrgId(s.getRootOrgId());
		OrgEntity rootOrg = orgRepo.findOne(s.getRootOrgId());
		info.setRootOrgName(rootOrg.getName());
		info.setSecurityPhone(s.getSecurityPhone());
		info.setStudentCode(s.getStudentCode());

		return info;
	}

	@Override
	public List<Long> unbindStudentCode(Long rootOrgId, String studentCode, String identityNumber) {

		if (null == rootOrgId) {
			throw new StatusException("B-120001", "rootOrgId is null");
		}

		List<StudentEntity> studentList = Lists.newArrayList();

		StudentEntity s1 = null;
		StudentEntity s2 = null;
		if (null != studentCode) {
			s1 = studentRepo.findByStudentCodeAndRootOrgId(studentCode, rootOrgId);
			if (null != s1) {
				studentList.add(s1);
			}
		}
		if (null != identityNumber) {
			s2 = studentRepo.findByIdentityNumberAndRootOrgId(identityNumber, rootOrgId);
			if (null != s2) {
				if (null == s1 || !s1.getId().equals(s2.getId())) {
					studentList.add(s2);
				}
			}
		}

		List<Long> studentIdList = Lists.newArrayList();

		for (StudentEntity cur : studentList) {
			cur.setStudentCode(null);
			StudentEntity saved = studentRepo.save(cur);
			studentIdList.add(saved.getId());

			SyncStudentReq req = new SyncStudentReq();
			req.setEnable(saved.getEnable());
			req.setId(saved.getId());
			req.setIdentityNumber(saved.getIdentityNumber());
			req.setName(saved.getName());

			req.setOrgId(saved.getOrgId());
			OrgEntity org = orgRepo.findOne(saved.getOrgId());
			req.setOrgCode(org.getCode());
			req.setOrgName(org.getName());

			req.setPhoneNumber(saved.getPhoneNumber());
			req.setPhotoPath(saved.getPhotoPath());
			req.setRootOrgId(saved.getRootOrgId());
			req.setSecurityPhone(saved.getSecurityPhone());
			req.setStudentCode(saved.getStudentCode());

			req.setSyncType("update");

			dataSyncCloudService.syncStudent(req);
		}

		return studentIdList;
	}

	@Override
	public Boolean hasBeBound(String securityPhone) {

		StudentEntity s = studentRepo.findBySecurityPhone(securityPhone);
		return null != s;
	}

}
